type flBinPkg {
  build: Build
  cpvb: String!
  repo: String!
  url: String
  files: [flContentFile!]!
}

type flContentFile {
  binpkg: flBinPkg!
  path: String!
  timestamp: DateTime!
  size: Int!
}

extend type Query {
  flSearch(key: String!, machine: String): [flContentFile!]!
  flSearchV2(key: String!, machines: [String!]): [flContentFile!]!
  flCount(machine: String, buildId: String): Int!
  flList(machine: String!, buildId: String!, cpvb: String!): [flContentFile!]!
  flListPackages(machine: String!, buildId: String!): [flBinPkg!]!
}
