query search($key: String!, $machine: String) {
  flSearch(key: $key, machine: $machine) {
    path
    size
    timestamp
    binpkg {
      cpvb
      build {
        machine
        id
      }
    }
  }
}
